package com.zyzf.coffeepreorder.database

import android.content.Context
import androidx.room.Database
import androidx.room.Room
import androidx.room.RoomDatabase
import androidx.sqlite.db.SupportSQLiteDatabase
import com.zyzf.coffeepreorder.database.dao.CartDao
import com.zyzf.coffeepreorder.database.dao.CoffeeDao
import com.zyzf.coffeepreorder.database.dao.UserDao
import com.zyzf.coffeepreorder.database.model.Cart
import com.zyzf.coffeepreorder.database.model.Coffee
import com.zyzf.coffeepreorder.database.model.User
import com.zyzf.coffeepreorder.database.model.UserLogined
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

@Database(entities = [User::class, Coffee::class, Cart::class, UserLogined::class], version = 1, exportSchema = false)
abstract class AppDatabase : RoomDatabase() {
    abstract fun userDao(): UserDao
    abstract fun coffeeDao(): CoffeeDao
    abstract fun cartDao(): CartDao

    companion object {
        private const val DB_NAME: String = "coffee-preorder"

        @Volatile
        private var INSTANCE: AppDatabase? = null

        private suspend fun populateDatabase() {
            INSTANCE?.let { database ->
                // Users
                val userDao = database.userDao()
                val user1 = User("zyzf", "Ян К.", "+79911152503", "250303zyzf", "admin")
                userDao.insert(user1)
                // Coffees
                val coffeeDao = database.coffeeDao()
                val coffee1 = Coffee("Coffee1", 200.0, "Ing1", null, 0)
                val coffee2 = Coffee("Coffee2", 200.0, "Ing1", null, 0)
                val coffee3 = Coffee("Coffee3", 300.0, "Ing1", null, 0)
                val coffee4 = Coffee("Coffee4", 200.0, "Ing1", null, 0)
                val coffee5 = Coffee("Coffee5", 200.0, "Ing1", null, 0)
                val coffee6 = Coffee("Coffee6", 25.0, "Ing1", null, 0)
                val coffee7 = Coffee("Coffee7", 400.0, "Ing1", null, 0)
                val coffee8 = Coffee("Coffee8", 200.0, "Ing1", null, 0)
                val coffee9 = Coffee("Coffee9", 200.0, "Ing1", null, 0)
                val coffee10 = Coffee("Coffee10", 900.0, "Ing1", null, 0)
                val coffee11 = Coffee("Coffee11", 200.0, "Ing1", null, 0)
                coffeeDao.insert(coffee1.name, coffee1.cost, coffee1.ingredients)
                coffeeDao.insert(coffee2.name, coffee2.cost, coffee2.ingredients)
                coffeeDao.insert(coffee3.name, coffee3.cost, coffee3.ingredients)
                coffeeDao.insert(coffee4.name, coffee4.cost, coffee4.ingredients)
                coffeeDao.insert(coffee5.name, coffee5.cost, coffee5.ingredients)
                coffeeDao.insert(coffee6.name, coffee6.cost, coffee6.ingredients)
                coffeeDao.insert(coffee7.name, coffee7.cost, coffee7.ingredients)
                coffeeDao.insert(coffee8.name, coffee8.cost, coffee8.ingredients)
                coffeeDao.insert(coffee9.name, coffee9.cost, coffee9.ingredients)
                coffeeDao.insert(coffee10.name, coffee10.cost, coffee10.ingredients)
                coffeeDao.insert(coffee11.name, coffee11.cost, coffee11.ingredients)
                // Cart
                val cartDao = database.cartDao()
                val cart = Cart()
                cartDao.insert(cart)
            }
        }

        fun getInstance(appContext: Context): AppDatabase {
            return INSTANCE ?: synchronized(this) {
                Room.databaseBuilder(
                    appContext,
                    AppDatabase::class.java,
                    DB_NAME
                )
                    .addCallback(object : Callback() {
                        override fun onCreate(db: SupportSQLiteDatabase) {
                            super.onCreate(db)
                            CoroutineScope(Dispatchers.IO).launch {
                                populateDatabase()
                            }
                        }
                    })
                    .build()
                    .also { INSTANCE = it }
            }
        }
    }
}