package com.zyzf.coffeepreorder.database.dao

import androidx.room.Dao
import androidx.room.Delete
import androidx.room.Query
import com.zyzf.coffeepreorder.database.model.Coffee
import com.zyzf.coffeepreorder.database.model.CoffeeWithCart
import kotlinx.coroutines.flow.Flow

@Dao
interface CoffeeDao {
    @Query("select * from coffee order by name collate nocase asc")
    fun getAll(): Flow<List<Coffee>>

    @Query("select * from coffee where cart_id is not null and count > 0 order by name collate nocase asc")
    fun getAllInCart(): Flow<List<Coffee>>

    @Query("select sum(cost) from coffee where cart_id is not null and count > 0")
    fun getSumInCart(): Double

    @Query("select coffee.uid, name, cost, ingredients, cart_id, count, cart.uid as cart_uid from coffee left join cart on coffee.cart_id = cart.uid where coffee.uid = :uid")
    suspend fun getByUid(uid: Int): CoffeeWithCart?

    @Query("insert into coffee (name, cost, ingredients, count) values (:name, :cost, :ingredients, 0)")
    suspend fun insert(name: String, cost: Double, ingredients: String): Long

    @Query("update coffee set name = :name, cost = :cost, ingredients = :ingredients, cart_id = :cartId, count = :count where uid = :uid")
    suspend fun update(uid: Int, name: String, cost: Double, ingredients: String, cartId: Int?, count: Int): Int?

    @Delete
    suspend fun delete(coffee: Coffee)
}