package com.zyzf.coffeepreorder.database.dao

import androidx.room.Dao
import androidx.room.Insert
import androidx.room.Query
import com.zyzf.coffeepreorder.database.model.User
import kotlinx.coroutines.flow.Flow

@Dao
interface UserDao {
    @Query("select * from user")
    fun getAll(): Flow<List<User>>

    @Query("select * from user where login = :login and password = :password")
    suspend fun tryLogin(login: String, password: String): User?

    @Query("select * from user where uid = :uid")
    suspend fun getByUid(uid: Int): User?

    @Query("select user.uid, login, fio, phone, password, role from user join user_logined on user_logined.user_id = user.uid limit 1")
    suspend fun getLogined(): User?

    @Query("insert into user_logined (user_id) values (:userId)")
    suspend fun setLogined(userId: Int)

    @Query("delete from user_logined")
    suspend fun logout()

    @Insert
    suspend fun insert(user: User)

    @Query("update user set login = :login, fio = :fio, phone = :phone, password = :password where uid = :uid")
    suspend fun update(uid: Int, login: String, fio: String, phone: String, password: String) : Int?

    @Query("delete from user where uid = :uid")
    suspend fun delete(uid: Int)
}