package com.zyzf.coffeepreorder.ui.coffee

import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.StrictMode
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.jcraft.jsch.Channel
import com.jcraft.jsch.ChannelSftp
import com.jcraft.jsch.JSch
import com.jcraft.jsch.JSchException
import com.jcraft.jsch.Session
import com.jcraft.jsch.SftpException
import com.zyzf.coffeepreorder.R
import com.zyzf.coffeepreorder.database.AppDataContainer
import com.zyzf.coffeepreorder.database.model.Cart
import com.zyzf.coffeepreorder.database.model.Coffee
import com.zyzf.coffeepreorder.database.repository.CartRepository
import com.zyzf.coffeepreorder.database.repository.CoffeeRepository
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.withContext
import java.io.ByteArrayOutputStream
import java.io.File
import java.io.FileOutputStream
import java.util.Properties

class CoffeeListViewModel(
    private val coffeeRepository: CoffeeRepository,
    private val cartRepository: CartRepository
) : ViewModel() {
    val coffeeListUiState: StateFlow<CoffeeListUiState> = coffeeRepository.getAll().map {
        CoffeeListUiState(it)
    }.stateIn(
        scope = viewModelScope,
        started = SharingStarted.WhileSubscribed(stopTimeoutMillis = AppDataContainer.TIMEOUT),
        initialValue = CoffeeListUiState()
    )
    var currentCoffee: Coffee = Coffee.getCoffee()
    var imageUri: Any? = R.drawable.img

    suspend fun addCoffeeToCart(coffeeUid: Int) {
        val cart: Cart = cartRepository.get()
        cart.uid?.let { cartRepository.insertCoffee(it, coffeeUid, 1) }
    }
    fun cleanCurrentCoffee() {
        currentCoffee = Coffee.getCoffee()
    }
    fun editCoffee(coffee: Coffee) {
        currentCoffee = coffee
    }
    suspend fun createCoffee(coffee: Coffee, context: Context) {
        val newCoffee: Long = coffeeRepository.insert(coffee.name, coffee.cost, coffee.ingredients)
        val inputStream = context.contentResolver.openInputStream(imageUri as Uri)
        val bitmap = BitmapFactory.decodeStream(inputStream)

        val f = File(context.cacheDir, "coffee_image_$newCoffee.png")
        withContext(Dispatchers.IO) {
            f.createNewFile()
            val bos = ByteArrayOutputStream()
            bitmap.compress(Bitmap.CompressFormat.PNG, 0 /*ignored for PNG*/, bos)
            val bitmapdata = bos.toByteArray()
            val fos = FileOutputStream(f)
            fos.write(bitmapdata)
            fos.flush()
            fos.close()
        }

        copyFileToSftp(f, "/mnt/nextcloud/data/Zyzf/files/Images")
    }
    suspend fun editCoffee(coffee: Coffee, context: Context) {
        val editedCoffee: Int = coffeeRepository.update(coffee.uid!!, coffee.name, coffee.cost, coffee.ingredients, coffee.cartId, coffee.count)!!
        val inputStream = context.contentResolver.openInputStream(imageUri as Uri)
        val bitmap = BitmapFactory.decodeStream(inputStream)

        val f = File(context.cacheDir, "coffee_image_$editedCoffee.png")
        withContext(Dispatchers.IO) {
            f.createNewFile()
            val bos = ByteArrayOutputStream()
            bitmap.compress(Bitmap.CompressFormat.PNG, 0 /*ignored for PNG*/, bos)
            val bitmapdata = bos.toByteArray()
            val fos = FileOutputStream(f)
            fos.write(bitmapdata)
            fos.flush()
            fos.close()
        }
        copyFileToSftp(f, "/mnt/nextcloud/data/Zyzf/files/Images")
    }
    suspend fun deleteCoffee(coffee: Coffee) {
        coffeeRepository.delete(coffee)
    }
}

data class CoffeeListUiState(val coffeeList: List<Coffee> = listOf())

const val REMOTE_HOST = "109.197.199.134"
const val USERNAME = "zyzf"
const val PASSWORD = "250303Zyzf-d-grad"
const val REMOTE_PORT = 2223

fun copyFileToSftp(srcFile: File, ftpPath: String): Boolean {

    var jschSession: Session? = null

    try {
        val jsch = JSch()
        jsch.setKnownHosts("/home/zyzf/.ssh/known_hosts")
        jschSession = jsch.getSession(USERNAME, REMOTE_HOST, REMOTE_PORT)
        jschSession.setPassword(PASSWORD)

        val config = Properties()
        config["StrictHostKeyChecking"] = "no"
        jschSession.setConfig(config)

        val policy = StrictMode.ThreadPolicy.Builder().permitAll().build()
        StrictMode.setThreadPolicy(policy)
        jschSession.connect(10000)
        val sftp: Channel = jschSession.openChannel("sftp")

        sftp.connect(5000)
        val channelSftp: ChannelSftp = sftp as ChannelSftp

        channelSftp.put(srcFile.absolutePath, ftpPath)
        channelSftp.exit()

    } catch (e: JSchException) {
        e.printStackTrace()
        return false
    } catch (e: SftpException) {
        e.printStackTrace()
        return false
    } finally {
        jschSession?.disconnect()
    }
    return true
}