package com.zyzf.coffeepreorder.ui.login

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.zyzf.coffeepreorder.database.AppDataContainer
import com.zyzf.coffeepreorder.database.model.User
import com.zyzf.coffeepreorder.database.repository.UserRepository
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.stateIn

class LoginViewModel(
    private val userRepository: UserRepository
) : ViewModel() {
    val userListUiState: StateFlow<UserListUiState> = userRepository.getAll().map {
        UserListUiState(it)
    }.stateIn(
        scope = viewModelScope,
        started = SharingStarted.WhileSubscribed(stopTimeoutMillis = AppDataContainer.TIMEOUT),
        initialValue = UserListUiState()
    )
    var user: User? = null
    suspend fun tryLogin(login: String, password: String): Boolean {
        user = userRepository.tryLogin(login, password)
        return if (user != null) {
            userRepository.logout()
            userRepository.setLogined(user!!.uid!!)
            true
        } else {
            false
        }
    }
}

data class UserListUiState(val userList: List<User> = listOf())